angular.module('stashd.cards', ['stashd.utils'])

.config(function($stateProvider, $urlRouterProvider) {

})
.value('cardsCache',[])
.directive('noScroll', function($document) {
    return {
        restrict: 'A',
        link: function($scope, $element, $attr) {

            $document.on('touchmove', function(e) {
                e.preventDefault();
            });
        }
    }
})
//Below analytics needs to be much smarter/better. Should be able to 'update' products, and add to
// numbers. Need to also 'track' movement better. Track js movement?
.factory('cardsApi', ['$http', '$ionicLoading', '$timeout', 'CONFIG', function($http, $ionicLoading, $timeout, CONFIG) {
    var apiUrl = CONFIG.API_URL + '/list/';
    var getApiData = function() {
        return $http({
            method: 'GET',
            url: apiUrl
        });
    };
    return {
        getApiData: getApiData,
    };
}])
.controller('CardsCtrl', ['$scope', '$rootScope', '$http', '$state', '$q', '$ionicLoading', '$ionicModal', 'TDCardDelegate', 'cardsApi', '$timeout', '$element', 'djangoAuth', 'cardsCache', 'TranslationsService',
    function($scope, $rootScope, $http, $state, $q, $ionicLoading, $ionicModal, TDCardDelegate, cardsApi, $timeout, $element, djangoAuth, cardsCache, TranslationsService) {
        TranslationsService.reloadFromServer();
        
        console.log('CARDS CTRL - cardsCache : ', cardsCache.length);
        var MIN_CARDS = 10;

        

        var loaderTemplate = '<div class="loader-img loader-img-cards"></div>'+
                            '<div class="loader-back"></div>';

        var showLoader = function(content){
            console.log('showing loader');
            $ionicLoading.show({
                template: loaderTemplate,
                animation: 'fade-in',
                showBackdrop: true,
                maxWidth: 200,
                showDelay: 0
            });
        }

        showLoader('fetch more!');
        var fetchInProgress = false;
        var fetchAPIData = function() {
            console.log('fetchAPIData called');
            if(!fetchInProgress){
                fetchInProgress = true;
                cardsApi.getApiData().then(function(result) {
                    console.log('fetchedCards',result);
                    // var cards = response.data;
                    if(result.data.length == 0){
                        $ionicLoading.hide();
                        fetchInProgress = false;
                        return;
                    }
                    for (var i = 0; i < result.data.length; i++) {
                        var c = document.createElement('img');
                        c.src = result.data[i].image;
                        c.onload = function(available){
                            $scope.cards.unshift(available);
                            $ionicLoading.hide();
                            //if last one and still there's 
                            if(i==result.data.length-1){
                                
                            }
                        }.bind(null, result.data[i]);

                        c.addEventListener('error', function(errorImg){
                            // to check for those imgs which are not loaded
                        }.bind(null, result.data[i]));

                        // var dummyEl = document.getElementById('dummy-image-card');
                        // if(dummyEl !== null){
                        //     dummyEl.innerHTML = '';
                        //     dummyEl.appendChild(c);
                        // }
                    }
                    fetchInProgress = false;
                    //below is to ensure that if all newly loaded items has no image, we want to trigger another batch of items straight away.
                    setTimeout(function(){
                        fetchMoreIfNeeded();    
                    },3000);
                }, function(err){
                    fetchInProgress = false;
                    if(err.status==403){
                        console.log('status 403');
                        $ionicLoading.hide();
                        djangoAuth.logout();
                    } else if(err.status==401){
                        console.log('status 401');
                        $ionicLoading.hide();
                        djangoAuth.logout();
                    }else{
                        console.log('unknown error',err);
                        $ionicLoading.hide();
                    }
                });
            }
        }

        $scope.cardDestroyed = function(index) {
            console.log('destroyed in cards.js at index : ' + index);
            console.log('we now have ' + $scope.cards.length + ' cards');
            $scope.cards.splice(index, 1);
            fetchMoreIfNeeded();
        };

        function fetchMoreIfNeeded() {
            if ($scope.cards.length < MIN_CARDS) {
                console.log('we need more cards, fetching more');
                if($scope.cards.length == 0){
                    showLoader('fetch more!');
                }
                fetchAPIData();
            } else{
                $ionicLoading.hide();
            }
        }
        $scope.addCard = function() {
            var newCard = $scope.cards[$scope.cards.length];
            $scope.cards.push(angular.extend({}, newCard));
        };
        var stressTestCounter = 0;
        function cardFetchingStressTest(counter) {
            $ionicLoading.hide();
            var startTime = Date.now();
            console.log('sending fetching order(' + counter + ')');
            cardsApi.getApiData().then(function(result) {
                console.log('fetched cards(' + counter + ') : ' + (Date.now()-startTime)/1000 + ' secs');
            }, function(err){
                if(err.status==403){
                    console.log('fetching error status 403(' + counter + ')' + (Date.now()-startTime)/1000 + ' secs');
                    djangoAuth.logout();
                } else if(err.status==401){
                    console.log('fetching error status 401(' + counter + ')' + (Date.now()-startTime)/1000 + ' secs');
                    djangoAuth.logout();
                }else{
                    console.log('fetching error unknown error(' + counter + ')' + (Date.now()-startTime)/1000 + ' secs',err);
                }
            });
            if(counter==10){
                return;
            }
            setTimeout(function(){
                stressTestCounter ++;
                cardFetchingStressTest(stressTestCounter);
            },2000);
        }
        setTimeout(function(){
            $scope.cards = cardsCache;
            fetchMoreIfNeeded();
            //cardFetchingStressTest(stressTestCounter);
        },400);
    }
])
.directive('ngIf', function() {
    return {
        link: function(scope, element, attrs) {
            if(scope.$eval(attrs.ngIf)) {
                // remove '<div ng-if...></div>'
                //element.replaceWith(element.children())
            } else {
                element.replaceWith(' ');
            }
        }
    }
});