'use strict';
angular.module('stashd.stylistshop', [])
.value('stylistCardsCache',[])
    .directive('noScroll', '$document',
        //[ '$document', function($document){}]
        function($document) {
            return {
                restrict: 'A',
                link: function($scope, $element, $attr) {

                    $document.on('touchmove', function(e) {
                        e.preventDefault();
                    });
                }
            }
        }
    )
.factory('stylistcardsApi', ['$http', 'CONFIG', function($http, CONFIG) {

    var BASE_URL = CONFIG.API_URL + '/list/?tag=';

    var getApiData = function (stylist_id) {
        return $http.get(BASE_URL + stylist_id);
    };
    
    return {
        getApiData: getApiData
    };

}])

.controller('stylistCardsCtrl', ['$scope', '$stateParams', '$log', 'stylistcardsApi', 'UserService', '$ionicLoading', 'stylistCardsCache',
    function ($scope, $stateParams, $log, stylistcardsApi, UserService, $ionicLoading, stylistCardsCache) {

    var MIN_stylistcards = 10;

    var stylist_id = $stateParams.stylist_id;


    // <======  Rewrite with accounts preferences
    var loaderTemplate = '<div class="loader-img loader-img-stylistshop"></div>'+
                            '<div class="loader-back"></div>';

    var showLoader = function(content){
            $ionicLoading.show({
                template: loaderTemplate,
                animation: 'fade-in',
                showBackdrop: true,
                maxWidth: 200,
                showDelay: 0
            });
        }
    var fetchInProgress = false;
    var fetchAPIData = function() {
    if(!fetchInProgress){
                fetchInProgress = true;
        stylistcardsApi.getApiData(stylist_id)
            .then(function(result) {
                if(result.data.length == 0){
                    $ionicLoading.hide();
                    fetchInProgress = false;
                    return;
                }
                for (var i = 0; i < result.data.length; i++) {
                    var c = document.createElement('img');
                    c.src = result.data[i].image;

                    c.onload = function(available){
                        $scope.cards.unshift(available);
                        $ionicLoading.hide();
                    }.bind(null, result.data[i]);

                    c.addEventListener('error', function(errorImg){
                        // to check for those imgs which are not loaded
                    }.bind(null, result.data[i]));

                    // var dummyEl = document.getElementById('dummy-image-card');
                    // if(dummyEl !== null){
                    //     dummyEl.innerHTML = '';
                    //     dummyEl.appendChild(c);
                    // }
                }
                fetchInProgress = false;
                    //below is to ensure that if all newly loaded items has no image, we want to trigger another batch of items straight away.
                    setTimeout(function(){
                        fetchMoreIfNeeded();    
                    },3000);
            }, function(err){
                fetchInProgress = false;
                    if(err.status==403){
                        console.log('status 403');
                        $ionicLoading.hide();
                        djangoAuth.logout();
                    } else if(err.status==401){
                        console.log('status 401');
                        $ionicLoading.hide();
                        djangoAuth.logout();
                    }else{
                        console.log('unknown error',err);
                        $ionicLoading.hide();
                    }
                });
    }
    }

    $scope.cardDestroyed = function(index) {
        $scope.cards.splice(index, 1);
        fetchMoreIfNeeded();
    };

    function fetchMoreIfNeeded() {
        if ($scope.cards.length < MIN_stylistcards) {
            if($scope.cards.length == 0){
                showLoader('fetch more!');
            }
            fetchAPIData();
        } else{
            $ionicLoading.hide();
        }
    }

    $scope.addCard = function() {
        var newCard = $scope.cards[$scope.cards.length];
        $scope.cards.push(angular.extend({}, newCard));
    };
    setTimeout(function(){
        $scope.cards = stylistCardsCache;
        fetchMoreIfNeeded();
    },400);

}]);
