'use strict';

angular.module('stashd.login', [])


//.factory('UserFactory', ['$http',
.factory('facebookConnectPlugin', ['$http', 'CONFIG',
    function($http, CONFIG) {
        var apiUrl = CONFIG.API_URL + '/accounts/facebook/login';

        var getAccountData = function() {
            return $http.get(apiUrl)
                .then(function(data) {
                    return data;
                });
        };

        return {
            getAccountData: getAccountData
        };
    }
])
.service('RegisterService', function($http, $log) {
        var userData;
        
        var setUserData = function(email, password){
            console.log('setting user data email ' + email + ', password ' + password);
            userData = {'email': email, 'password': password};
        };
        var getUserData = function(){
            return userData;
        }
        return {
            getUserData: getUserData,
            setUserData: setUserData
        };
    }
)
.service('UserService', ['$http', '$log', 'CONFIG',
    function($http, $log, CONFIG) {
        // Store user data on ionic local storage
        var setUser = function(user_data) {
            window.localStorage.starter_facebook_user = JSON.stringify(user_data);
            $log.info(user_data);
            var userApi = CONFIG.API_URL + '/user/';

             $http.post(userApi + JSON.stringify(user_data))
                     .then(function successCallback(response) {
                         $log.log(success);
              }, function errorCallback(err) {
                    console.log('set user error', err);
                    $log.error(err);
                })
        };

        var getUser = function(){
            return JSON.parse(window.localStorage.starter_facebook_user || '{}');
        };

        return {
            getUser: getUser,
            setUser: setUser
        };
    }
])

.controller('LoginController', function($http, $scope, $state, $q, UserService, $log,RegisterService, djangoAuth, $ionicHistory, cardsCache, stylistCardsCache, storeCardsCache, TranslationsService, ServerLocatorService) {
    if(ServerLocatorService.getCurrentCountryCode() == 'CN'){
        $('.hide-for-chinese').hide();
    } else{
        $('.hide-for-chinese').show();
    }
    $scope.translations = TranslationsService.getData();
    console.log('$scope.translations',$scope.translations);
    //StatusBar.hide(); ### Need to install ngCordova Status Bar
    console.log('isAuth in Login?',djangoAuth.authenticated);
    $scope.$parent.$parent.pageClass = 'login-page';
    $scope.email = '';
    $scope.password = '';
    $scope.buttonDisabled = false;
    if(window.localStorage.getItem('token') != null) //window.localStorage.getItem('uid') != null ||
    {
        $http.defaults.headers.common.Authorization = 'Token ' + window.localStorage.getItem('token');        
        $state.go('cards');
    }
    else
    {
        // This is the success callback from the login method
        var fbLoginSuccess = function(response) {
            // $ionicLoading.show({
            //     template: 'Logging in...'
            // });
            // console.log('f1');
            $scope.buttonDisabled = true;
            if (!response.authResponse){
                // console.log('f2');
                $scope.buttonDisabled = false;
                fbLoginError("Cannot find the authResponse");
                // $ionicLoading.hide();
                return;
                
            }
            var authResponse = response.authResponse;
            // console.log('f3');
            getFacebookProfileInfo(authResponse).then(function(profileInfo) {
                // console.log('f4');
                djangoAuth.fb_login(authResponse.accessToken,authResponse.userID);
                // $scope.buttonDisabled = false;
                // $ionicLoading.hide();
            }, function(fail){
                // console.log('f5');
                $scope.buttonDisabled = false;
                // $ionicLoading.hide();
            });
        };

        // This is the fail callback from the login method
        var fbLoginError = function(error){
            $scope.buttonDisabled = false;
            $console.log('fbLoginError', error);
            // $ionicLoading.hide();
        };
        
        //following enforces the clear cache in case user logs in with another user and the cards view cache is cleared here.
        $scope.$on("$ionicView.enter", function () {
            console.log('clearing cache');
            $ionicHistory.clearCache();
            $ionicHistory.clearHistory();
            cardsCache.length = 0;
            stylistCardsCache.length = 0;
            storeCardsCache.length = 0;
        });

        // This method is to get the user profile info from the facebook api
        var getFacebookProfileInfo = function (authResponse) {
            var info = $q.defer();
            facebookConnectPlugin.api('/me?fields=email,name&access_token=' + authResponse.accessToken, null,
                function (response) {
                    console.log(response);
                    info.resolve(response);
                },
                function (response) {
                    console.log(response);
                    info.reject(response);
                }
            );
            return info.promise;
        };
    
        $scope.eventlog = function(){
            if(typeof window.plugins !== "undefined" && typeof window.plugins.flurry !== "undefined") window.plugins.flurry.logEvent('UserVisit');
        }
        
        $scope.create_account = function(formData){
            //Validate.form_validation(formData,$scope.errors);
            if(!this.password || this.password.length < 6){
                return;
            }
            RegisterService.setUserData(this.email,this.password);
        $state.go('registeropt');
        }

        //This method is executed when the user press the "Login with facebook" button
        $scope.facebookSignIn = function() {
            //window.plugins.flurry.logEvent('UserVisit');
            if(window.cordova){
                $scope.buttonDisabled = true;
                facebookConnectPlugin.getLoginStatus(function(success){
                        // $ionicLoading.show({
                        //     template: 'Logging in...'
                        // });

                        // Ask the permissions
                        facebookConnectPlugin.login(['email', 'public_profile'], fbLoginSuccess, fbLoginError);
                });
            } else{
                //Facebook Login not available
            }
        };
    }
})

.directive('loginView', ['$document',
    function($document) {
        return {
            restrict: 'E',
            templateUrl: 'app/core/login/login.html',
            link: function($scope, $element, $attr) {
            }
        }

    }
])

.controller('IntroCtrl', function($scope, $state, $ionicSlideBoxDelegate) {
 
  // Called to navigate to the main app
  $scope.startApp = function() {
    $state.go('main');
  };
  $scope.next = function() {
    $ionicSlideBoxDelegate.next();
  };
  $scope.previous = function() {
    $ionicSlideBoxDelegate.previous();
  };

  // Called each time the slide changes
  $scope.slideChanged = function(index) {
    $scope.slideIndex = index;
  };
})

.controller('MainCtrl', function($scope, $state) {
  console.log('MainCtrl');
  
  $scope.toIntro = function(){
    $state.go('cards');
  }
});


