'use strict';

angular.module('angularDjangoRegistrationAuthApp',['ngCookies'])
  .service('djangoAuth', function djangoAuth($q, $http, $cookies, $rootScope, CONFIG, $state) {
    // AngularJS will instantiate a singleton by calling "new" on this function
    var service = {
        /* START CUSTOMIZATION HERE */
        // Change this to point to your Django REST Auth API
        // e.g. /api/rest-auth  (DO NOT INCLUDE ENDING SLASH)
        //'API_URL': CONFIG.API_URL + '/rest-auth',
        // Set use_session to true to use Django sessions to store security token.
        // Set use_session to false to store the security token locally and transmit it as a custom header.
        'use_session': false,
        /* END OF CUSTOMIZATION */
        'authenticated': null,
        'authPromise': null,
        'request': function(args) {
            // Let's retrieve the token from the cookie, if available
            if($cookies.token){
                console.log('exiting token found');
                $http.defaults.headers.common.Authorization = 'Token ' + $cookies.token;
            }
            // Continue
            params = args.params || {}
            args = args || {};
            var deferred = $q.defer(),
                url = CONFIG.API_URL + '/rest-auth' + args.url,
                method = args.method || "GET",
                params = params,
                data = args.data || {};
            // Fire the request, as configured.
            $http({
                url: url,
                withCredentials: this.use_session,
                method: method.toUpperCase(),
                headers: {'X-CSRFToken': $cookies['csrftoken'],'Content-Type':'application/json'},
                params: params,
                data: data
            })
            .success(angular.bind(this,function(data, status, headers, config) {
                deferred.resolve(data, status);
            }))
            .error(angular.bind(this,function(data, status, headers, config) {
                console.log("error syncing with: " + url);
                // Set request status
                if(data){
                    data.status = status;
                }
                if(status == 0){
                    if(data == ""){
                        data = {};
                        data['status'] = 0;
                        data['non_field_errors'] = ["Could not connect. Please try again."];
                    }
                    // or if the data is null, then there was a timeout.
                    if(data == null){
                        // Inject a non field error alerting the user
                        // that there's been a timeout error.
                        data = {};
                        data['status'] = 0;
                        data['non_field_errors'] = ["Server timed out. Please try again."];
                    }
                }
                deferred.reject(data, status, headers, config);
            }));
            return deferred.promise;
        },
        'register': function(password1,password2,email,more){
            
            var data = {
                'password1':password1,
                'password2':password2,
                'email':email
            }
            data = angular.extend(data,more);
            console.log('calling register api with data : ', data);
            return this.request({
                'method': "POST",
                'url': "/registration/",
                'data' :data
            });
        },
        'login': function(email,password){            
            var djangoAuth = this;
            return this.request({
                'method': "POST",
                'url': "/login/",
                'data':{
                    'email':email,
                    'password':password
                }
            }).then(function(data){
                console.log('after login data (' + CONFIG.API_URL + ')',data);
                if(!djangoAuth.use_session){
                    $http.defaults.headers.common.Authorization = 'Token ' + data.key;
                    $cookies.token = data.key;
                }
                $http.defaults.headers.common.Authorization = 'Token ' + data.key;
                window.localStorage.setItem('token', data.key);
                djangoAuth.authenticated = true;
                $rootScope.$broadcast("djangoAuth.logged_in", data);
                //$state.go('cards');
            });
        },
        'fb_login': function(access_token,code){
            //var djangoAuth = this;
            return this.request({
                'method': "POST",
                'url': "/facebook/",
                'data':{
                    'access_token':access_token,
                    'code':code
                }
            }).then(function(data){
                console.log('after fb login data',data);
                $http.defaults.headers.common.Authorization = 'Token ' + data.key;
                window.localStorage.setItem('token', data.key);
                djangoAuth.authenticated = true;
                $rootScope.$broadcast("djangoAuth.logged_in", data);
                $state.go('cards');
            },function(err){
                console.log('fb_login error',err);
            });
        },
        'logout': function(){
            var djangoAuth = this;
            return this.request({
                'method': "POST",
                'url': "/logout/"
            }).then(function(data){
                console.log('logging out..');
                delete $http.defaults.headers.common.Authorization;
                delete $cookies.token;
                window.localStorage.removeItem('token');
                window.localStorage.removeItem('uid');
                djangoAuth.authenticated = false;
                $rootScope.$broadcast("djangoAuth.logged_out");
                $state.go('login');
            }, function(err){
                console.log('server logout failed but logging out on app anyway..');
                delete $http.defaults.headers.common.Authorization;
                delete $cookies.token;
                window.localStorage.removeItem('token');
                window.localStorage.removeItem('uid');
                djangoAuth.authenticated = false;
                $rootScope.$broadcast("djangoAuth.logged_out");
                $state.go('login');
            });
        },
        'changePassword': function(password1,password2){
            return this.request({
                'method': "POST",
                'url': "/password/change/",
                'data':{
                    'new_password1':password1,
                    'new_password2':password2
                }
            });
        },
        'resetPassword': function(email){
            return this.request({
                'method': "POST",
                'url': "/password/reset/",
                'data':{
                    'email':email
                }
            });
        },
        'profile': function(){
            return this.request({
                'method': "GET",
                'url': "/user/"
            }); 
        },
        'updateProfile': function(data){
            return this.request({
                'method': "PATCH",
                'url': "/user/",
                'data':data
            }); 
        },
        'verify': function(key){
            return this.request({
                'method': "POST",
                'url': "/registration/verify-email/",
                'data': {'key': key} 
            });            
        },
        'confirmReset': function(uid,token,password1,password2){
            return this.request({
                'method': "POST",
                'url': "/password/reset/confirm/",
                'data':{
                    'uid': uid,
                    'token': token,
                    'new_password1':password1,
                    'new_password2':password2
                }
            });
        },
        'authenticationStatus': function(restrict, force){
            // Set restrict to true to reject the promise if not logged in
            // Set to false or omit to resolve when status is known
            // Set force to true to ignore stored value and query API
            restrict = restrict || false;
            force = force || false;
            if(this.authPromise == null || force){
                this.authPromise = this.request({
                    'method': "GET",
                    'url': "/user/"
                })
            }
            var da = this;
            var getAuthStatus = $q.defer();
            console.log('f1');
            if(this.authenticated != null && !force){
                console.log('f2');
                // We have a stored value which means we can pass it back right away.
                if(this.authenticated == false && restrict){
                    getAuthStatus.reject("User is not logged in.");
                }else{
                    getAuthStatus.resolve();
                }
            }else{
                console.log('f3');
                // There isn't a stored value, or we're forcing a request back to
                // the API to get the authentication status.
                this.authPromise.then(function(){
                    console.log('f4');
                    da.authenticated = true;
                    getAuthStatus.resolve();
                },function(){
                    console.log('f5');
                    da.authenticated = false;
                    if(restrict){
                        getAuthStatus.reject("User is not logged in.");
                    }else{
                        getAuthStatus.resolve();
                    }
                });
            }
            return getAuthStatus.promise;
        },
        'initialize': function(url, sessions){
            //this.API_URL = url;
            this.use_session = sessions;
            return this.authenticationStatus();
        }

    }
    return service;
  });
